@@ -1,29 +1,25 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2004 Sam Lantinga
+    Copyright (C) 1997-2012 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
-
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_events_c.h,v 1.7 2005/08/21 06:18:54 slouken Exp $";
-#endif
+#include "SDL_config.h"
 
 /* Useful functions and variables from SDL_events.c */
 #include "SDL_events.h"
@@ -33,8 +29,8 @@ extern int SDL_StartEventLoop(Uint32 flags);
 extern void SDL_StopEventLoop(void);
 extern void SDL_QuitInterrupt(void);
 
-extern void SDL_Lock_EventThread();
-extern void SDL_Unlock_EventThread();
+extern void SDL_Lock_EventThread(void);
+extern void SDL_Unlock_EventThread(void);
 extern Uint32 SDL_EventThreadID(void);
 
 /* Event handler init routines */
@@ -68,6 +64,9 @@ extern int SDL_PrivateExpose(void);
 extern int SDL_PrivateQuit(void);
 extern int SDL_PrivateSysWMEvent(SDL_SysWMmsg *message);
 
+/* Used to clamp the mouse coordinates separately from the video surface */
+extern void SDL_SetMouseRange(int maxX, int maxY);
+
 /* Used by the activity event handler to remove mouse focus */
 extern void SDL_ResetMouse(void);
 
