@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2006 Sam Lantinga
+    Copyright (C) 1997-2012 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,7 @@
 #include "SDL_rwops.h"
 
 
-#if defined(__WIN32__)
+#if defined(__WIN32__) && !defined(__SYMBIAN32__)
 
 /* Functions to read/write Win32 API file pointers */
 /* Will not use it on WinCE because stdio is buffered, it means
@@ -43,6 +43,8 @@
 #define INVALID_SET_FILE_POINTER 0xFFFFFFFF
 #endif
 
+#define READAHEAD_BUFFER_SIZE	1024
+
 static int SDLCALL win32_file_open(SDL_RWops *context, const char *filename, const char *mode)
 {
 #ifndef _WIN32_WCE
@@ -54,9 +56,12 @@ static int SDLCALL win32_file_open(SDL_RWops *context, const char *filename, con
 	int		a_mode;
 
 	if (!context)
-		return -1;
+		return -1; /* failed (invalid call) */
 		
 	context->hidden.win32io.h = INVALID_HANDLE_VALUE; /* mark this as unusable */
+	context->hidden.win32io.buffer.data = NULL;
+	context->hidden.win32io.buffer.size = 0;
+	context->hidden.win32io.buffer.left = 0;
 
 	/* "r" = reading, file must exist */
 	/* "w" = writing, truncate existing, file may not exist */
@@ -72,7 +77,13 @@ static int SDLCALL win32_file_open(SDL_RWops *context, const char *filename, con
 	w_right    = ( a_mode || SDL_strchr(mode,'+') || truncate ) ? GENERIC_WRITE : 0;
 
 	if (!r_right && !w_right) /* inconsistent mode */
-		return -1; /* failed (invalid call)*/
+		return -1; /* failed (invalid call) */
+
+	context->hidden.win32io.buffer.data = (char *)SDL_malloc(READAHEAD_BUFFER_SIZE);
+	if (!context->hidden.win32io.buffer.data) {
+		SDL_OutOfMemory();
+		return -1;
+	}
 
 #ifdef _WIN32_WCE
 	{
@@ -80,8 +91,10 @@ static int SDLCALL win32_file_open(SDL_RWops *context, const char *filename, con
 		wchar_t *filenameW = SDL_stack_alloc(wchar_t, size);
 
 		if ( MultiByteToWideChar(CP_UTF8, 0, filename, -1, filenameW, size) == 0 ) {
-			SDL_SetError("Unable to convert filename to Unicode");
 			SDL_stack_free(filenameW);
+			SDL_free(context->hidden.win32io.buffer.data);
+			context->hidden.win32io.buffer.data = NULL;
+			SDL_SetError("Unable to convert filename to Unicode");
 			return -1;
 		}
 		h = CreateFile(filenameW, (w_right|r_right), (w_right)? 0 : FILE_SHARE_READ, 
@@ -89,23 +102,85 @@ static int SDLCALL win32_file_open(SDL_RWops *context, const char *filename, con
 		SDL_stack_free(filenameW);
 	}
 #else
-	/* Do not open a dialog box if failure */
-	old_error_mode = SetErrorMode(SEM_NOOPENFILEERRORBOX|SEM_FAILCRITICALERRORS);
+	{
 
-	h = CreateFile(filename, (w_right|r_right), (w_right)? 0 : FILE_SHARE_READ, 
-		           NULL, (must_exist|truncate|a_mode), FILE_ATTRIBUTE_NORMAL,NULL);
+	/* handle Unicode filenames.  We do some tapdancing here to make sure this
+	   works on Win9x, which doesn't support anything but 1-byte codepages. */
+	const size_t size = SDL_strlen(filename)+1;
+	static int unicode_support = -1;
 
-	/* restore old behaviour */
-	SetErrorMode(old_error_mode);
+	if (unicode_support == -1) {
+		OSVERSIONINFO osVerInfo;     /* Information about the OS */
+		osVerInfo.dwOSVersionInfoSize = sizeof(osVerInfo);
+		if (!GetVersionEx(&osVerInfo)) {
+			unicode_support = 0;
+		} else if (osVerInfo.dwPlatformId != VER_PLATFORM_WIN32_WINDOWS) {
+			unicode_support = 1;  /* Not Win95/98/ME. */
+		} else {
+			unicode_support = 0;
+		}
+	}
+
+	if (unicode_support) {  /* everything but Win95/98/ME. */
+		wchar_t *filenameW = SDL_stack_alloc(wchar_t, size);
+		if ( MultiByteToWideChar(CP_UTF8, 0, filename, -1, filenameW, size) == 0 ) {
+			SDL_stack_free(filenameW);
+			SDL_free(context->hidden.win32io.buffer.data);
+			context->hidden.win32io.buffer.data = NULL;
+			SDL_SetError("Unable to convert filename to Unicode");
+			return -1;
+		}
+
+		/* Do not open a dialog box if failure */
+		old_error_mode = SetErrorMode(SEM_NOOPENFILEERRORBOX|SEM_FAILCRITICALERRORS);
+		h = CreateFileW(filenameW, (w_right|r_right), (w_right)? 0 : FILE_SHARE_READ,
+					   NULL, (must_exist|truncate|a_mode), FILE_ATTRIBUTE_NORMAL,NULL);
+		/* restore old behaviour */
+		SetErrorMode(old_error_mode);
+
+		SDL_stack_free(filenameW);
+	} else {
+		/* CP_UTF8 might not be supported (Win95), so use SDL_iconv to get wchars. */
+		/* Use UCS2: no UTF-16 support here. Try again in SDL 1.3.  :) */
+		char *utf16 = SDL_iconv_string("UCS2", "UTF8", filename, SDL_strlen(filename) + 1);
+		char *filenameA = SDL_stack_alloc(char, size * 6);  /* 6, just in case. */
+		BOOL bDefCharUsed = FALSE;
+
+		/* Dither down to a codepage and hope for the best. */
+		if (!utf16 ||
+			!WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)utf16, -1, filenameA, size*6, 0, &bDefCharUsed) ||
+			bDefCharUsed) {
+			SDL_stack_free(filenameA);
+			SDL_free(utf16);
+			SDL_free(context->hidden.win32io.buffer.data);
+			context->hidden.win32io.buffer.data = NULL;
+			SDL_SetError("Unable to convert filename to Unicode");
+			return -1;
+		}
+
+		/* Do not open a dialog box if failure */
+		old_error_mode = SetErrorMode(SEM_NOOPENFILEERRORBOX|SEM_FAILCRITICALERRORS);
+		h = CreateFile(filenameA, (w_right|r_right), (w_right)? 0 : FILE_SHARE_READ,
+		               NULL, (must_exist|truncate|a_mode), FILE_ATTRIBUTE_NORMAL,NULL);
+		/* restore old behaviour */
+		SetErrorMode(old_error_mode);
+
+		SDL_stack_free(filenameA);
+		SDL_free(utf16);
+	}
+
+	}
 #endif /* _WIN32_WCE */
 
 	if (h==INVALID_HANDLE_VALUE) {
+		SDL_free(context->hidden.win32io.buffer.data);
+		context->hidden.win32io.buffer.data = NULL;
 		SDL_SetError("Couldn't open %s",filename);
 		return -2; /* failed (CreateFile) */
 	}
 	context->hidden.win32io.h = h;
 	context->hidden.win32io.append = a_mode;
-	
+
 	return 0; /* ok */
 }
 static int SDLCALL win32_file_seek(SDL_RWops *context, int offset, int whence)
@@ -117,7 +192,13 @@ static int SDLCALL win32_file_seek(SDL_RWops *context, int offset, int whence)
 		SDL_SetError("win32_file_seek: invalid context/file not opened");
 		return -1;
 	}
-	
+
+	/* FIXME: We may be able to satisfy the seek within buffered data */
+	if (whence == RW_SEEK_CUR && context->hidden.win32io.buffer.left) {
+		offset -= context->hidden.win32io.buffer.left;
+    }
+    context->hidden.win32io.buffer.left = 0;
+
 	switch (whence) {
 		case RW_SEEK_SET:		
 			win32whence = FILE_BEGIN; break;
@@ -129,7 +210,7 @@ static int SDLCALL win32_file_seek(SDL_RWops *context, int offset, int whence)
 			SDL_SetError("win32_file_seek: Unknown value for 'whence'");			
 			return -1;
 	}
-	
+
 	file_pos = SetFilePointer(context->hidden.win32io.h,offset,NULL,win32whence);
 
 	if ( file_pos != INVALID_SET_FILE_POINTER )
@@ -140,21 +221,50 @@ static int SDLCALL win32_file_seek(SDL_RWops *context, int offset, int whence)
 }
 static int SDLCALL win32_file_read(SDL_RWops *context, void *ptr, int size, int maxnum)
 {
+	int		total_need; 
+	int		total_read = 0; 
+    int     read_ahead;
+	DWORD	byte_read;
 	
-	int		total_bytes; 
-	DWORD	byte_read,nread;
+	total_need = size*maxnum;
 	
-	total_bytes = size*maxnum;
-	
-	if (!context || context->hidden.win32io.h == INVALID_HANDLE_VALUE || total_bytes<=0 || !size) 	
+	if (!context || context->hidden.win32io.h == INVALID_HANDLE_VALUE || total_need<=0 || !size) 	
 		return 0;
-	
-	if (!ReadFile(context->hidden.win32io.h,ptr,total_bytes,&byte_read,NULL)) {
-		SDL_Error(SDL_EFREAD);
-		return 0;
-	}
-	nread = byte_read/size;
-	return nread;
+
+    if (context->hidden.win32io.buffer.left > 0) {
+        void *data = (char *)context->hidden.win32io.buffer.data +
+                             context->hidden.win32io.buffer.size -
+                             context->hidden.win32io.buffer.left;
+        read_ahead = SDL_min(total_need, context->hidden.win32io.buffer.left); 
+        SDL_memcpy(ptr, data, read_ahead);
+        context->hidden.win32io.buffer.left -= read_ahead;
+
+        if (read_ahead == total_need) {
+            return maxnum;
+        }
+        ptr = (char *)ptr + read_ahead;
+        total_need -= read_ahead;       
+		total_read += read_ahead;
+    }
+
+    if (total_need < READAHEAD_BUFFER_SIZE) {
+        if (!ReadFile(context->hidden.win32io.h,context->hidden.win32io.buffer.data,READAHEAD_BUFFER_SIZE,&byte_read,NULL)) {
+            SDL_Error(SDL_EFREAD);
+            return 0;
+        }
+        read_ahead = SDL_min(total_need, (int)byte_read);
+        SDL_memcpy(ptr, context->hidden.win32io.buffer.data, read_ahead);
+        context->hidden.win32io.buffer.size = byte_read;
+        context->hidden.win32io.buffer.left = byte_read-read_ahead;
+        total_read += read_ahead;
+    } else {
+        if (!ReadFile(context->hidden.win32io.h,ptr,total_need,&byte_read,NULL)) {
+            SDL_Error(SDL_EFREAD);
+            return 0;
+        }
+        total_read += byte_read;
+    }
+	return (total_read/size);
 }
 static int SDLCALL win32_file_write(SDL_RWops *context, const void *ptr, int size, int num)
 {
@@ -167,6 +277,11 @@ static int SDLCALL win32_file_write(SDL_RWops *context, const void *ptr, int siz
 	if (!context || context->hidden.win32io.h==INVALID_HANDLE_VALUE || total_bytes<=0 || !size) 	
 		return 0;
 
+	if (context->hidden.win32io.buffer.left) {
+		SetFilePointer(context->hidden.win32io.h,-context->hidden.win32io.buffer.left,NULL,FILE_CURRENT);
+		context->hidden.win32io.buffer.left = 0;
+	}
+
 	/* if in append mode, we must go to the EOF before write */
 	if (context->hidden.win32io.append) {
 		if ( SetFilePointer(context->hidden.win32io.h,0L,NULL,FILE_END) == INVALID_SET_FILE_POINTER ) {
@@ -191,6 +306,10 @@ static int SDLCALL win32_file_close(SDL_RWops *context)
 			CloseHandle(context->hidden.win32io.h);
 			context->hidden.win32io.h = INVALID_HANDLE_VALUE; /* to be sure */
 		}
+		if (context->hidden.win32io.buffer.data) {
+			SDL_free(context->hidden.win32io.buffer.data);
+			context->hidden.win32io.buffer.data = NULL;
+		}
 		SDL_FreeRW(context);
 	}
 	return(0);
@@ -366,18 +485,18 @@ SDL_RWops *SDL_RWFromFile(const char *file, const char *mode)
 	SDL_RWops *rwops = NULL;
 #ifdef HAVE_STDIO_H
 	FILE *fp = NULL;
+	(void) fp;
 #endif
 	if ( !file || !*file || !mode || !*mode ) {
 		SDL_SetError("SDL_RWFromFile(): No file or no mode specified");
 		return NULL;
 	}
 
-#if defined(__WIN32__)
+#if defined(__WIN32__) && !defined(__SYMBIAN32__)
 	rwops = SDL_AllocRW();
 	if (!rwops)
 		return NULL; /* SDL_SetError already setup by SDL_AllocRW() */
-	rwops->hidden.win32io.h = INVALID_HANDLE_VALUE;
-	if (win32_file_open(rwops,file,mode)) {
+	if (win32_file_open(rwops,file,mode) < 0) {
 		SDL_FreeRW(rwops);
 		return NULL;
 	}	
