@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2003  Sam Lantinga
+    Copyright (C) 1997-2012  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,11 +22,25 @@
 #include "SDL_config.h"
 
 #include "SDL_QuartzVideo.h"
+#include "SDL_QuartzWM.h"
 
 #include <IOKit/IOMessage.h> /* For wake from sleep detection */
 #include <IOKit/pwr_mgt/IOPMLib.h> /* For wake from sleep detection */
 #include "SDL_QuartzKeys.h"
 
+/*
+ * On Leopard, this is missing from the 64-bit headers
+ */
+#if defined(__LP64__) && !defined(__POWER__)
+/*
+ * Workaround for a bug in the 10.5 SDK: By accident, OSService.h does
+ * not include Power.h at all when compiling in 64bit mode. This has
+ * been fixed in 10.6, but for 10.5, we manually define UsrActivity
+ * to ensure compilation works.
+ */
+#define UsrActivity 1
+#endif
+
 /* 
  * In Panther, this header defines device dependent masks for 
  * right side keys. These definitions only exist in Panther, but
@@ -64,10 +78,10 @@
 #endif
 
 void     QZ_InitOSKeymap (_THIS) {
-    const void *KCHRPtr;
+    BOOL saw_layout = NO;
     UInt32 state;
     UInt32 value;
-    int i;
+    Uint16 i;
     int world = SDLK_WORLD_0;
 
     for ( i=0; i<SDL_TABLESIZE(keymap); ++i )
@@ -87,9 +101,14 @@ void     QZ_InitOSKeymap (_THIS) {
     keymap[QZ_F10] = SDLK_F10;
     keymap[QZ_F11] = SDLK_F11;
     keymap[QZ_F12] = SDLK_F12;
+    keymap[QZ_F13] = SDLK_F13;
+    keymap[QZ_F14] = SDLK_F14;
+    keymap[QZ_F15] = SDLK_F15;
+/*
     keymap[QZ_PRINT] = SDLK_PRINT;
     keymap[QZ_SCROLLOCK] = SDLK_SCROLLOCK;
     keymap[QZ_PAUSE] = SDLK_PAUSE;
+*/
     keymap[QZ_POWER] = SDLK_POWER;
     keymap[QZ_BACKQUOTE] = SDLK_BACKQUOTE;
     keymap[QZ_1] = SDLK_1;
@@ -193,31 +212,96 @@ void     QZ_InitOSKeymap (_THIS) {
         why we keep the static table, too.
      */
 
-    /* Get a pointer to the systems cached KCHR */
-    KCHRPtr = (void *)GetScriptManagerVariable(smKCHRCache);
-    if (KCHRPtr)
-    {
-        /* Loop over all 127 possible scan codes */
-        for (i = 0; i < 0x7F; i++)
-        {
-            /* We pretend a clean start to begin with (i.e. no dead keys active */
-            state = 0;
+#if (MAC_OS_X_VERSION_MAX_ALLOWED >= 1050)
+    if (TISCopyCurrentKeyboardLayoutInputSource != NULL) {
+        TISInputSourceRef src = TISCopyCurrentKeyboardLayoutInputSource();
+        if (src != NULL) {
+            CFDataRef data = (CFDataRef)
+                TISGetInputSourceProperty(src,
+                    kTISPropertyUnicodeKeyLayoutData);
+            if (data != NULL) {
+                const UCKeyboardLayout *layout = (const UCKeyboardLayout *)
+                    CFDataGetBytePtr(data);
+                if (layout != NULL) {
+                    const UInt32 kbdtype = LMGetKbdType();
+                    saw_layout = YES;
 
-            /* Now translate the key code to a key value */
-            value = KeyTranslate(KCHRPtr, i, &state) & 0xff;
+                    /* Loop over all 127 possible scan codes */
+                    for (i = 0; i < 0x7F; i++) {
+                        UniChar buf[16];
+                        UniCharCount count = 0;
 
-            /* If the state become 0, it was a dead key. We need to translate again,
-                passing in the new state, to get the actual key value */
-            if (state != 0)
-                value = KeyTranslate(KCHRPtr, i, &state) & 0xff;
+                        /* We pretend a clean start to begin with (i.e. no dead keys active */
+                        state = 0;
 
-            /* Now we should have an ascii value, or 0. Try to figure out to which SDL symbol it maps */
-            if (value >= 128)     /* Some non-ASCII char, map it to SDLK_WORLD_* */
-                keymap[i] = world++;
-            else if (value >= 32)     /* non-control ASCII char */
-                keymap[i] = value;
+                        if (UCKeyTranslate(layout, i, kUCKeyActionDown, 0, kbdtype,
+                                           0, &state, 16, &count, buf) != noErr) {
+                            continue;
+                        }
+
+                        /* If the state become 0, it was a dead key. We need to
+                           translate again, passing in the new state, to get
+                           the actual key value */
+                        if (state != 0) {
+                            if (UCKeyTranslate(layout, i, kUCKeyActionDown, 0, kbdtype,
+                                               0, &state, 16, &count, buf) != noErr) {
+                                continue;
+                            }
+                        }
+
+                        if (count != 1) {
+                            continue;  /* no multi-char. Use SDL 1.3 instead. :) */
+                        }
+
+                        value = (UInt32) buf[0];
+                        if (value >= 128) {
+                            /* Some non-ASCII char, map it to SDLK_WORLD_* */
+                            if (world < 0xFF) {
+                                keymap[i] = world++;
+                            }
+                        } else if (value >= 32) {     /* non-control ASCII char */
+                            keymap[i] = value;
+                        }
+                    }
+                }
+            }
+            CFRelease(src);
         }
     }
+#endif
+
+#if (MAC_OS_X_VERSION_MIN_REQUIRED < 1050)
+    if (!saw_layout) {
+        /* Get a pointer to the systems cached KCHR */
+        const void *KCHRPtr = (const void *)GetScriptManagerVariable(smKCHRCache);
+        if (KCHRPtr)
+        {
+            /* Loop over all 127 possible scan codes */
+            for (i = 0; i < 0x7F; i++)
+            {
+                /* We pretend a clean start to begin with (i.e. no dead keys active */
+                state = 0;
+
+                /* Now translate the key code to a key value */
+                value = KeyTranslate(KCHRPtr, i, &state) & 0xff;
+
+                /* If the state become 0, it was a dead key. We need to translate again,
+                    passing in the new state, to get the actual key value */
+                if (state != 0)
+                    value = KeyTranslate(KCHRPtr, i, &state) & 0xff;
+
+                /* Now we should have an ascii value, or 0. Try to figure out to which SDL symbol it maps */
+                if (value >= 128) {     /* Some non-ASCII char, map it to SDLK_WORLD_* */
+                    if (world < 0xFF) {
+                        keymap[i] = world++;
+                    }
+                } else if (value >= 32) {     /* non-control ASCII char */
+                    keymap[i] = value;
+                }
+            }
+        }
+    }
+#endif
 
     /* 
         The keypad codes are re-setup here, because the loop above cannot
@@ -248,8 +332,8 @@ void     QZ_InitOSKeymap (_THIS) {
 
 static void QZ_DoKey (_THIS, int state, NSEvent *event) {
 
-    NSString *chars;
-    unsigned int numChars;
+    NSString *chars = NULL;
+    unsigned int i, numChars;
     SDL_keysym key;
     
     /* 
@@ -260,19 +344,17 @@ static void QZ_DoKey (_THIS, int state, NSEvent *event) {
         contains multiple characters, we'll use 0 as
         the scancode/keysym.
     */
-    chars = [ event characters ];
-    numChars = [ chars length ];
-
-    if (numChars == 1) {
-
-        key.scancode = [ event keyCode ];
-        key.sym      = keymap [ key.scancode ];
-        key.unicode  = [ chars characterAtIndex:0 ];
-        key.mod      = KMOD_NONE;
-
-        SDL_PrivateKeyboard (state, &key);
+    if (SDL_TranslateUNICODE && state == SDL_PRESSED) {
+        [field_edit interpretKeyEvents:[NSArray arrayWithObject:event]];
+        chars = [ event characters ];
+        numChars = [ chars length ];
+        if (numChars > 0)
+            [field_edit setString:@""];
+    } else {
+        numChars = 0;
     }
-    else if (numChars == 0) {
+
+    if (numChars == 0) {
       
         key.scancode = [ event keyCode ];
         key.sym      = keymap [ key.scancode ];
@@ -281,10 +363,16 @@ static void QZ_DoKey (_THIS, int state, NSEvent *event) {
 
         SDL_PrivateKeyboard (state, &key);
     }
-    else /* (numChars > 1) */ {
+    else if (numChars >= 1) {
+
+        key.scancode = [ event keyCode ];
+        key.sym      = keymap [ key.scancode ];
+        key.unicode  = [ chars characterAtIndex:0 ];
+        key.mod      = KMOD_NONE;
+
+        SDL_PrivateKeyboard (state, &key);
       
-        int i;
-        for (i = 0; i < numChars; i++) {
+        for (i = 1; i < numChars; i++) {
 
             key.scancode = 0;
             key.sym      = 0;
@@ -617,10 +705,8 @@ static void QZ_GetMouseLocation (_THIS, NSPoint *p) {
 void QZ_DoActivate (_THIS) {
 
     SDL_PrivateAppActive (1, SDL_APPINPUTFOCUS | (QZ_IsMouseInWindow (this) ? SDL_APPMOUSEFOCUS : 0));
-    
-    /* Hide the cursor if it was hidden by SDL_ShowCursor() */
-    if (!cursor_should_be_visible)
-        QZ_HideMouse (this);
+
+    QZ_UpdateCursor(this);
 
     /* Regrab input, only if it was previously grabbed */
     if ( current_grab_mode == SDL_GRAB_ON ) {
@@ -635,6 +721,8 @@ void QZ_DoActivate (_THIS) {
         QZ_GetMouseLocation (this, &p);
         SDL_PrivateMouseMotion (0, 0, p.x, p.y);
     }
+
+    QZ_UpdateCursor(this);
 }
 
 void QZ_DoDeactivate (_THIS) {
@@ -647,9 +735,7 @@ void QZ_DoDeactivate (_THIS) {
     /* Reassociate mouse and cursor */
     CGAssociateMouseAndMouseCursorPosition (1);
 
-    /* Show the cursor if it was hidden by SDL_ShowCursor() */
-    if (!cursor_should_be_visible)
-        QZ_ShowMouse (this);
+    QZ_UpdateCursor(this);
 }
 
 void QZ_SleepNotificationHandler (void * refcon,
@@ -711,10 +797,7 @@ static int QZ_OtherMouseButtonToSDL(int button)
 
 void QZ_PumpEvents (_THIS)
 {
-    static Uint32 screensaverTicks = 0;
-    Uint32 nowTicks;
-    int firstMouseEvent;
-    CGMouseDelta dx, dy;
+    int32_t dx, dy;
 
     NSDate *distantPast;
     NSEvent *event;
@@ -725,11 +808,14 @@ void QZ_PumpEvents (_THIS)
         return;  /* don't do anything if there's no screen surface. */
 
     /* Update activity every five seconds to prevent screensaver. --ryan. */
-    nowTicks = SDL_GetTicks();
-    if ((nowTicks - screensaverTicks) > 5000)
-    {
-        UpdateSystemActivity(UsrActivity);
-        screensaverTicks = nowTicks;
+    if (!allow_screensaver) {
+        static Uint32 screensaverTicks;
+        Uint32 nowTicks = SDL_GetTicks();
+        if ((nowTicks - screensaverTicks) > 5000)
+        {
+            UpdateSystemActivity(UsrActivity);
+            screensaverTicks = nowTicks;
+        }
     }
 
     pool = [ [ NSAutoreleasePool alloc ] init ];
@@ -737,10 +823,7 @@ void QZ_PumpEvents (_THIS)
 
     winRect = NSMakeRect (0, 0, SDL_VideoSurface->w, SDL_VideoSurface->h);
     
-    /* send the first mouse event in absolute coordinates */
-    firstMouseEvent = 1;
-    
-    /* accumulate any additional mouse moved events into one SDL mouse event */
+    /* while grabbed, accumulate all mouse moved events into one SDL mouse event */
     dx = 0;
     dy = 0;
     
@@ -837,34 +920,27 @@ void QZ_PumpEvents (_THIS)
                             so we have to call the lowlevel window server
                             function. This is less accurate but works OK.                         
                         */
-                        CGMouseDelta dx1, dy1;
+                        int32_t dx1, dy1;
                         CGGetLastMouseDelta (&dx1, &dy1);
                         dx += dx1;
                         dy += dy1;
                     }
-                    else if (firstMouseEvent) {
+                    else {
                         
                         /*
-                            Get the first mouse event in a possible
-                            sequence of mouse moved events. Since we
-                            use absolute coordinates, this serves to
-                            compensate any inaccuracy in deltas, and
-                            provides the first known mouse position,
-                            since everything after this uses deltas
+                            Get the absolute mouse location. This is not the
+                            mouse location after the currently processed event,
+                            but the *current* mouse location, i.e. after all
+                            pending events. This means that if there are
+                            multiple mouse moved events in the queue, we make
+                            multiple identical calls to SDL_PrivateMouseMotion(),
+                            but that's no problem since the latter only
+                            generates SDL events for nonzero movements. In my
+                            experience on PBG4/10.4.8, this rarely happens anyway.
                         */
                         NSPoint p;
                         QZ_GetMouseLocation (this, &p);
                         SDL_PrivateMouseMotion (0, 0, p.x, p.y);
-                        firstMouseEvent = 0;
-                   }
-                    else {
-                    
-                        /*
-                            Get the amount moved since the last drag or move event,
-                            add it on for one big move event at the end.
-                        */
-                        dx += [ event deltaX ];
-                        dy += [ event deltaY ];
                     }
                     
                     /* 
@@ -872,8 +948,7 @@ void QZ_PumpEvents (_THIS)
                         into the game window. This still generates a mouse moved event,
                         but not as a result of the warp (so it's in the right direction).
                     */
-                    if ( grab_state == QZ_VISIBLE_GRAB &&
-                         !isInGameWin ) {
+                    if ( grab_state == QZ_VISIBLE_GRAB && !isInGameWin ) {
                        
                         NSPoint p;
                         QZ_GetMouseLocation (this, &p);
@@ -896,6 +971,7 @@ void QZ_PumpEvents (_THIS)
                     if ( !isInGameWin && (SDL_GetAppState() & SDL_APPMOUSEFOCUS) ) {
                     
                         SDL_PrivateAppActive (0, SDL_APPMOUSEFOCUS);
+
                         if (grab_state == QZ_INVISIBLE_GRAB)
                             /*The cursor has left the window even though it is
                               disassociated from the mouse (and therefore
@@ -911,15 +987,16 @@ void QZ_PumpEvents (_THIS)
                               cursor enters the window again, making it obvious
                               to the user that the grab is broken.*/
                             CGAssociateMouseAndMouseCursorPosition (1);
-                        if (!cursor_should_be_visible)
-                            QZ_ShowMouse (this);
+
+                        QZ_UpdateCursor(this);
                     }
                     else
                     if ( isInGameWin && (SDL_GetAppState() & (SDL_APPMOUSEFOCUS | SDL_APPINPUTFOCUS)) == SDL_APPINPUTFOCUS ) {
                     
                         SDL_PrivateAppActive (1, SDL_APPMOUSEFOCUS);
-                        if (!cursor_should_be_visible)
-                            QZ_HideMouse (this);
+
+                        QZ_UpdateCursor(this);
+
                         if (grab_state == QZ_INVISIBLE_GRAB) { /*see comment above*/
                             QZ_PrivateWarpCursor (this, SDL_VideoSurface->w / 2, SDL_VideoSurface->h / 2);
                             CGAssociateMouseAndMouseCursorPosition (0);
@@ -932,10 +1009,12 @@ void QZ_PumpEvents (_THIS)
                         Uint8 button;
                         dy = [ event deltaY ];
                         dx = [ event deltaX ];
-                        if ( dy > 0.0 || dx > 0.0 ) /* Scroll up */
+                        if ( dy > 0.0 ) /* Scroll up */
                             button = SDL_BUTTON_WHEELUP;
-                        else /* Scroll down */
+                        else if ( dy < 0.0 ) /* Scroll down */
                             button = SDL_BUTTON_WHEELDOWN;
+                        else
+                            break; /* Horizontal scroll */
                         /* For now, wheel is sent as a quick down+up */
                         SDL_PrivateMouseButton (SDL_PRESSED, button, 0, 0);
                         SDL_PrivateMouseButton (SDL_RELEASED, button, 0, 0);
@@ -949,7 +1028,16 @@ void QZ_PumpEvents (_THIS)
                     break;
                 case NSFlagsChanged:
                     break;
-                    /* case NSAppKitDefined: break; */
+                case NSAppKitDefined:
+                    [ NSApp sendEvent:event ];
+                    if ([ event subtype ] == NSApplicationActivatedEventType && (mode_flags & SDL_FULLSCREEN)) {
+                        /* the default handling of this event seems to reset any cursor set by [NSCursor set] (used by SDL_SetCursor() in fullscreen mode) to the default system arrow cursor */
+                        SDL_Cursor *sdlc = SDL_GetCursor();
+                        if (sdlc != NULL && sdlc->wm_cursor != NULL) {
+                            [ sdlc->wm_cursor->nscursor set ];
+                        }
+                    }
+                    break;
                     /* case NSApplicationDefined: break; */
                     /* case NSPeriodic: break; */
                     /* case NSCursorUpdate: break; */
